
// SearchGoogle.java
// Andrew Davison, May 2009, ad@fivedots.coe.psu.ac.th

/* Query Google's AJAX Search API using its REST-based interface that 
   returns search results in JSON format.
   For details, see
      http://code.google.com/apis/ajaxsearch/documentation/reference.html#_intro_fonje

  Use Google's image search to look for a book cover image for the title
  specified on the command line by the user.
  e.g:
      run SearchGoogle "Killer Game Programming in Java"
      
  The first four matches are listed (description and URL), and the first is 
  automatically saved as a PNG file called <fnm>Cover.png, where 
  fnm is the filename in the image's URL.
*/


import java.net.*;
import java.io.*;

import org.json.*;


public class SearchGoogle 
{
  public static void main(String args[])
  {       
    if (args.length == 0) {
      System.err.println("Supply a title");
      System.exit(1);
    }

    String imUrlStr = bookCoverSearch(args[0]);
    WebUtils.downloadImage(imUrlStr);
  }  // end of main()



  private static String bookCoverSearch(String title)
  // query Google's image search for "title" + "book cover"
  {
    System.out.println("Searching Google Image for the book title \"" + title + "\"");

    try {
      // Convert spaces to +, etc. to make a valid URL
      String ueTitle = URLEncoder.encode("\"" + title + "\"" + "\"book cover\"", "UTF-8"); 
      String urlStr = "http://ajax.googleapis.com/ajax/services/search/images?v=1.0" +
                        "&q=" + ueTitle;

      String jsonStr = WebUtils.webGetString(urlStr);

      JSONObject json = new JSONObject(jsonStr);
      WebUtils.saveString("temp.json", json.toString(2) );    // indent the string
      return getCoverURL(json);
    }
 	catch (Exception e) 
	{ System.out.println(e);  
      System.exit(1);
    }
    return null;
  }  // end of bookCoverSearch()



  private static String getCoverURL(JSONObject json)
  // return the first URL of a cover image found by the search
  {
    try {
      System.out.println("\nTotal no. of possible results: " +
                      json.getJSONObject("responseData")
                          .getJSONObject("cursor")
                          .getString("estimatedResultCount") + "\n");
                          
      
      // list the contents and URLs returned
      JSONArray ja = json.getJSONObject("responseData").getJSONArray("results");
      for (int i = 0; i < ja.length(); i++) {
        System.out.print((i+1) + ". ");
        JSONObject j = ja.getJSONObject(i);
        System.out.println("Content: " + j.getString("contentNoFormatting"));
        System.out.println("       URL: "+ j.getString("url") + "\n");
      }
      if (ja.length() > 0)
        return ja.getJSONObject(0).getString("url");   // return first URL
    }
    catch (Exception e) 
    { System.out.println(e); 
      System.exit(1);
    }
    
    return null;
  } // end of getCoverURL()
  

 } // end of SearchGoogle class
