
// SearchYahoo.java
// Andrew Davison, May 2009, ad@fivedots.coe.psu.ac.th

/* Use Yahoo's Product Search API to look for a book cover image for 
  the title specified on the command line by the user.
  e.g:
      run SearchYahoo "Killer Game Programming in Java"
      
  At most the first 5 matches are listed (title and picture URL), and the first 
  picture URL is downloaded and saved as a PNG file called <fnm>Cover.png, where 
  fnm is the filename in the image's URL.

  Details on the Product Search API can be found at:
       http://developer.yahoo.com/shopping/V3/productSearch.html
*/

import java.net.*;
import java.io.*;

import org.w3c.dom.*;
import javax.xml.xpath.*;



public class SearchYahoo 
{
  // **** CHANGE THIS LINE ****
  private static String APP_ID = "?????"; 

  private static XPath xpath;


  public static void main(String args[])
  {       
    if (args.length == 0) {
      System.err.println("Supply a title");
      System.exit(1);
    }
    xpath = createXPath(); 
    String imUrlStr = bookTitleSearch(args[0]);  
    WebUtils.downloadImage(imUrlStr);
  }  // end of main()



  private static XPath createXPath()
  // create a XPath object that understands the Yahoo Namespace
  {
    XPath xpath = XPathFactory.newInstance().newXPath();
    NamespaceContextImpl ns = new NamespaceContextImpl("yah", "urn:yahoo:prods");
    xpath.setNamespaceContext(ns);

    return xpath;
  }  // end of createXPath()



  private static String bookTitleSearch(String title)
  // query Yahoo for a book with the specified title, and return its image URL
  {
    System.out.println("Searching Yahoo for the book title \"" + title + "\"");
    try {
      String ueTitle = URLEncoder.encode(title, "UTF-8");

      String urlStr = "http://shopping.yahooapis.com/ShoppingService/V3/productSearch?" +
                          "appid=" + APP_ID + "&" + 
                          "department=56&" +     // limit search to book department
                          "query=" + ueTitle + "&" +
                          "results=5" ;          // at most 5 results

      Document doc = WebUtils.parse( WebUtils.webGet(urlStr) );
      WebUtils.saveDoc(doc, "temp.xml");

      return getPictureURL(doc);
    }
	catch (Exception e) 
	{ System.out.println(e); 
      System.exit(1);
    }

    return null;  
  }  // end of bookTitleSearch()




  private static String getPictureURL(Document doc)
  /* print matching titles and GridImage picture URLs, and 
     return the first picture URL */
  {
    try {
      String totalStr =      // get the total no. of results (the @totalResultsAvailable attribute)
          (String) xpath.evaluate("//yah:Products/@totalResultsAvailable", 
                                                       doc, XPathConstants.STRING);
      System.out.println("Total no. of products matches: " + totalStr);

      NodeList nodes = doc.getElementsByTagName("Product"); // get all the products
      String firstURL = null; 
      for (int i = 0; i < nodes.getLength(); i++) { 
        System.out.print( (i+1) + ". ");
        Element elem = (Element) nodes.item(i);
        System.out.println("Title: " + WebUtils.getElemText(elem, "ProductName"));
        String urlStr = getGridImageURL(elem); 
        System.out.println("   URL: " + urlStr);
        if (firstURL == null)  // store the first URL match that isn't null
          firstURL = urlStr;
      }
      return firstURL;
    }
    catch(Exception e)
    { System.out.println(e);  
      System.exit(1);
    }
    return null;
  }  // end of getPictureURL()


  private static String getGridImageURL(Element elem)
  // get the GridImage URL, if one exists
  {
    NodeList nodeList = elem.getElementsByTagName("GridImage");
    if (nodeList.getLength() > 0) {
     Element el = (Element) nodeList.item(0);
     return WebUtils.getElemText(el,"Url");
    }
    return null;
  }  // end of getGridImageURL()


 } // end of SearchYahoo class
